/*******************************************************************************
 *	TmsD.h	Produced by Bidl
 *******************************************************************************
 */

#ifndef TMSD_H
#define TMSD_H 1

#include <Boap.h>
#include <BObj.h>
#include <BDate.h>
#include <BTimeStamp.h>
#include <BComplex.h>
#include <BList.h>
#include <BArray.h>

//*****************************************************************************/
//	Base objects for BEAM TMS System
//			T.Barnaby,	BEAM Ltd,	2006-12-07
//*****************************************************************************/
//
namespace Tms {
	enum CyclePeriod 	{ CyclePeriodAll, CyclePeriodCalibration, CyclePeriodEvent0, CyclePeriodEvent1, CyclePeriodEvent2, CyclePeriodEvent3, CyclePeriodEvent4, CyclePeriodEvent5, CyclePeriodEvent6, CyclePeriodEvent7, CyclePeriodEvent8, CyclePeriodEvent9};

	enum DataType 	{ DataTypeRaw};

	enum DataFunction 	{ DataFunctionRaw, DataFunctionMean, DataFunctionMeanAll, DataFunctionMean0, DataFunctionMean1};

	enum TestOutput 	{ TestOutputFrefLocal, TestOutputPllL1, TestOutputPllL2};

	enum Priority 	{ PriorityLow, PriorityNormal, PriorityHigh};

	class NameValue {
	public:
		NameValue(BString pname = BString(), BString pvalue = BString());
	public:
		BString	name;	///< The Name of the value
		BString	value;	///< The actual value in string form
	};

	/// This class stores a Physical Pick-Up channel id
	class PuChannel {
	public:
		PuChannel(BUInt8 pmoduleNum = BUInt8(), BUInt8 ppupeNum = BUInt8(), BUInt8 ppupeChan = BUInt8());
	public:
		BUInt8	moduleNum;	///< The Module number
		BUInt8	pupeNum;	///< The PUPE number
		BUInt8	pupeChan;	///< The PUPE channel
	};

	/// This class stores the status of an individual Pick-Up
	class PuStatus {
	public:
		PuStatus(BInt32 prunning = BInt32(), BError perror = BError());
	public:
		BInt32	running;	///< The Pick-Up is currently running.
		BError	error;	///< The Pick-Up's current error status.
	};

	/// This class describes the configuration of the TMS
	class ConfigInfo {
	public:
		ConfigInfo(BArray<PuChannel > ppuReferences = BArray<PuChannel >());
	public:
		BArray<PuChannel >	puReferences;	///< The logical to physical Pick-Up table. Each PuReference includes a Module Controller identifier, a Physical Pick-Up number and a Physical Channel.
	};

	/// This class defines the data to be acquired and/or fetched
	class DataInfo {
	public:
		DataInfo(BUInt32 pcycleNumber = BUInt32(), BUInt32 pchannel = BUInt32(), BUInt32 pcyclePeriod = BUInt32(), BUInt32 pstartTime = BUInt32(), BUInt32 porbitNumber = BUInt32(), BUInt32 pbunchNumber = BUInt32(), BUInt32 pfunction = BUInt32(), BUInt32 pargument = BUInt32(), BUInt32 pnumValues = BUInt32(), BInt32 pbeyondPeriod = BInt32());
	public:
		BUInt32	cycleNumber;	///< The PS Cycle number
		BUInt32	channel;	///< The pick-up channel number
		BUInt32	cyclePeriod;	///< The cycle period the data is from
		BUInt32	startTime;	///< The start time in milli-seconds in the cycle period (starting from 0)
		BUInt32	orbitNumber;	///< The starting orbit number (starting from 0)
		BUInt32	bunchNumber;	///< The Bunch number (starting from 1 (0 is all bunches))
		BUInt32	function;	///< The data processing function to perform or performed. (0 normal data)
		BUInt32	argument;	///< The Argument to the data processing function
		BUInt32	numValues;	///< The total number of data points to return
		BInt32	beyondPeriod;	///< If set allows reads of data beyond the end of the period
	};

	/// This is the definition of a single data value
	class DataValue {
	public:
		DataValue(BInt16 psigma = BInt16(), BInt16 pdeltaX = BInt16(), BInt16 pdeltaY = BInt16(), BInt16 ptime = BInt16());
	public:
		BInt16	sigma;	///< The Sigma value
		BInt16	deltaX;	///< The DeltaX value
		BInt16	deltaY;	///< The DeltaY value
		BInt16	time;	///< The Time in ms this sample was processed
	};

	/// This class stores the raw data
	class Data {
	public:
		Data(BUInt32 pnumValues = BUInt32(), BUInt32 pdataType = BUInt32(), BUInt32 pnumBunches = BUInt32(), BUInt32 pnumChannels = BUInt32(), BArray<DataValue > pdataValues = BArray<DataValue >(), BArray<BError > perrors = BArray<BError >());
	public:
		BUInt32	numValues;	///< The total number of data samples
		BUInt32	dataType;	///< The type of data in the data block
		BUInt32	numBunches;	///< The number of bunches
		BUInt32	numChannels;	///< The number of channels
		BArray<DataValue >	dataValues;	///< The data
		BArray<BError >	errors;	///< Individual errors for each channel within dataValues
	};

	/// This class defines the Pick-Up state table
	class PuStateTable {
	public:
		PuStateTable(BUInt32 pperiod = BUInt32(), BUInt32 pstate = BUInt32(), BUInt32 pharmonic = BUInt32(), BUInt32 pnumBunches = BUInt32(), BUInt32 pbunchMask = BUInt32(), BArray<BUInt8 > pphaseTable = BArray<BUInt8 >());
	public:
		BUInt32	period;	///< The Cycle period this state is used for
		BUInt32	state;	///< The State table entry
		BUInt32	harmonic;	///< The harmonic number for this state
		BUInt32	numBunches;	///< The number of bunches to capture
		BUInt32	bunchMask;	///< Bitmask defining which buckets the bunches are captured from. Bit 0 is bucket 1, bit 1 is bucket 2 etc
		BArray<BUInt8 >	phaseTable;	///< The Phase table for this state
	};

	/// This class defines the parameters for a PS processing cycle
	class CycleParam {
	public:
		CycleParam(BString pcycleType = BString(), BString pname = BString(), BString pinfo = BString(), BUInt32 pring = BUInt32(), BUInt32 pchannel = BUInt32(), BUInt32 ppllCycleStartFrequency = BUInt32(), BUInt32 ppllInitialFrequency = BUInt32(), BUInt32 ppllInitialFrequencyDelay = BUInt32(), BUInt32 ppllFrefGain = BUInt32(), BUInt32 ppllGain = BUInt32(), BUInt32 ppllDdsMinimum = BUInt32(), BUInt32 ppllDdsMaximum = BUInt32(), BUInt32 pstateDelay = BUInt32(), BArray<BInt32 > pfrefPhaseDelay = BArray<BInt32 >(), BArray<PuStateTable > pstateTable = BArray<PuStateTable >(), BArray<BString > psettings = BArray<BString >());
	public:
		BString	cycleType;	///< The Cycle Type Name of this parameter set, normally the BEAM type the set of parameters is designed to measure
		BString	name;	///< The name of this set of parameters
		BString	info;	///< Information on this parameter set
		BUInt32	ring;	///< The ring this configuration is for, 0 defines all rings
		BUInt32	channel;	///< The channel number this configuration is for, 0 defines all channels
		BUInt32	pllCycleStartFrequency;	///< This defines the initial PLL frequency. This is loaded on START_CYCLE.
		BUInt32	pllInitialFrequency;	///< This defines the initial PLL frequency. This is loaded after the delay given in pllInitialFrequencyDelay.
		BUInt32	pllInitialFrequencyDelay;	///< This defines the delay in milliseconds from START_CYCLE when the pllInitialFrequency is loaded.
		BUInt32	pllFrefGain;	///< The gain the FREF signal. This is a value in the range +-8191. A normal value would be around 4096.
		BUInt32	pllGain;	///< The gain of the PLL feedback system. This is the gain reduction of the PLL in terms of right bit shifts. A bit shift of 7 is about unity gain.
		BUInt32	pllDdsMinimum;	///< PLL DDS minimum frequency. If this and pllDdsMaximum is set to 0, this feature is disabled.
		BUInt32	pllDdsMaximum;	///< PLL DDS maximum frequency. If this and pllDdsMinimum is set to 0, this feature is disabled.
		BUInt32	stateDelay;	///< The delay in FREF periods when in the delay state
		BArray<BInt32 >	frefPhaseDelay;	///< The phase delay parameters for the Fref timing signal for each of the Pick-Up channels. This is set based on the position of the Pick-Up's in the PS ring. Its value is based of Fref / 512.
		BArray<PuStateTable >	stateTable;	///< The array of State Table entries for the processing run.
		BArray<BString >	settings;	///< A string array defining the settings for the states. Used for CycleParam editors
	};

	// This class is used to return the list of CycleParm's
	class CycleParamItem {
	public:
		CycleParamItem(BString pcycleType = BString(), BUInt32 pring = BUInt32(), BUInt32 pchannel = BUInt32());
	public:
		BString	cycleType;	///< The Cycle Type Name of this parameter set, normally the BEAM type the set of parameters is designed to measure
		BUInt32	ring;	///< The ring number this configuration is for, 0 defines all rings
		BUInt32	channel;	///< The channel number this configuration is for, 0 defines all channels
	};

	/// This class defines the parameters for a test data capture
	class TestCaptureInfo {
	public:
		TestCaptureInfo(BUInt32 psource = BUInt32(), BUInt32 pclock = BUInt32(), BUInt32 pstartTime = BUInt32(), BUInt32 ppostTriggerDelay = BUInt32(), BUInt32 ptriggerMask = BUInt32(), BInt32 ptriggerAnd = BInt32(), BInt32 ptriggerStore = BInt32(), BInt32 ptriggerSourceData = BInt32(), BInt32 ptriggerStateEnable = BInt32(), BUInt32 ptriggerState = BUInt32());
	public:
		BUInt32	source;	///< The source data (0  - 3)
		BUInt32	clock;	///< The Clock source
		BUInt32	startTime;	///< The start time in ms from CYCLE_START before trigger is activated
		BUInt32	postTriggerDelay;	///< The delay, in clock cycles, after the trigger before capture starts
		BUInt32	triggerMask;	///< The Trigger bit mask. This is the bit mask of the 8 timing signals
		BInt32	triggerAnd;	///< The Trigger function is an AND rather than an OR
		BInt32	triggerStore;	///< Store the trigger in the upper 8 data bits
		BInt32	triggerSourceData;	///< Use lower 32bits of data as trigger source rather than timing signals
		BInt32	triggerStateEnable;	///< Trigger in state enable
		BUInt32	triggerState;	///< Trigger state number
	};

	// Special PUPE configuration for test purposes
	class PupeConfig {
	public:
		PupeConfig(BUInt32 pinternalTimingMask = BUInt32(), BInt32 pdoubleInjection = BInt32(), BInt32 padcSysclkSync = BInt32(), BInt32 pdisableBlr = BInt32());
	public:
		BUInt32	internalTimingMask;	///< Use internal, software/hardware generated, timing signals for the given signals.
		BInt32	doubleInjection;	///< Simulate double injection
		BInt32	adcSysclkSync;	///< Sets the ADC clock to be synchronised with the SYSCLK timing clock
		BInt32	disableBlr;	///< Disable the BLR algorithm
	};

	/// Cycle information
	class CycleInformationPeriod {
	public:
		CycleInformationPeriod(BUInt32 pcyclePeriod = BUInt32(), BUInt32 pstartTime = BUInt32(), BUInt32 pendTime = BUInt32(), BUInt32 pharmonic = BUInt32(), BUInt32 pnumBunches = BUInt32(), BUInt32 pbunchMask = BUInt32(), BUInt32 pnumValues = BUInt32());
	public:
		BUInt32	cyclePeriod;	///< The Cycle Period
		BUInt32	startTime;	///< The start time in ms
		BUInt32	endTime;	///< The end time in ms
		BUInt32	harmonic;	///< The Machines harmonic number
		BUInt32	numBunches;	///< The number of bunches
		BUInt32	bunchMask;	///< Bitmask defining which buckets the bunches are captured from. Bit 0 is bucket 1, bit 1 is bucket 2 etc
		BUInt32	numValues;	///< The total number of raw data values available
	};

	class CycleInformation {
	public:
		CycleInformation(BUInt32 pcycleNumber = BUInt32(), BString pcycleType = BString(), BList<CycleInformationPeriod > pperiods = BList<CycleInformationPeriod >());
	public:
		BUInt32	cycleNumber;	///< The PS Cycle number
		BString	cycleType;	///< The Cycle Type Name
		BList<CycleInformationPeriod >	periods;	///< The list of cycle periods
	};

	/// Cycle Type information
	class CycleTypeInformationPeriod {
	public:
		CycleTypeInformationPeriod(BUInt32 pcyclePeriod = BUInt32(), BUInt32 pharmonic = BUInt32(), BUInt32 pnumBunches = BUInt32(), BUInt32 pbunchMask = BUInt32());
	public:
		BUInt32	cyclePeriod;	///< The Cycle Period
		BUInt32	harmonic;	///< The Machines harmonic number
		BUInt32	numBunches;	///< The number of bunches
		BUInt32	bunchMask;	///< Bitmask defining which buckets the bunches are captured from. Bit 0 is bucket 1, bit 1 is bucket 2 etc
	};

	class CycleTypeInformation {
	public:
		CycleTypeInformation(BString pcycleType = BString(), BString pinfo = BString(), BList<CycleTypeInformationPeriod > pperiods = BList<CycleTypeInformationPeriod >());
	public:
		BString	cycleType;	///< The Cycle Type Name
		BString	info;	///< Information string on this cycle type
		BList<CycleTypeInformationPeriod >	periods;	///< The list of cycle periods
	};

	class Simulation {
	public:
		Simulation(BInt32 ptiming = BInt32(), BInt32 pdoubleInjection = BInt32(), BInt32 pdata = BInt32(), BInt32 psetNextCycle = BInt32(), BString pcycleType = BString());
	public:
		BInt32	timing;	///< Simulate timing signals
		BInt32	doubleInjection;	///< Simulate double injection
		BInt32	data;	///< Simulate FREF and Analogue Sigma, DeltaX and DeltaY data
		BInt32	setNextCycle;	///< Simulate setNextCycle
		BString	cycleType;	///< The cycle type to set
	};

	/// This interface provides functions to control, test and get statistics from an individual pick-up
	//
	/// This interface provides functions to configure and capture data from individual pick-up
	//
	/// This interface provides functions to control, test and get statistics from the TMS as a whole
	//
	/// This interface provides functions to capture data from the TMS as a whole
	//
	/// This interface provides functions for events to be sent to clients from the TMS as a whole
	//
}

#endif
